"""Logger and other default settings for the Delos client."""

from enum import Enum

from loguru import logger as delos_logger


class VerboseLevel(int, Enum):
    """Verbose level for the logger."""

    NONE = 0
    INFO = 1
    DEBUG = 2


__all__ = ["VerboseLevel", "delos_logger"]



from collections.abc import AsyncGenerator
from pathlib import Path
from typing import IO, Any, Literal

import requests
from _typeshed import Incomplete
from pydantic import BaseModel

from .endpoints import Endpoints as Endpoints
from .endpoints import RequestMethod as RequestMethod
from .exceptions import APIKeyMissingError as APIKeyMissingError
from .models import ResponseFormat
from .releases import AllReleases as AllReleases
from .settings import VerboseLevel as VerboseLevel
from .settings import delos_logger as delos_logger

COSMOS_PLATFORM_BACKEND_URL: str

class APIClient(BaseModel):
    api_key: str
    _server_url: str
    _verbose: VerboseLevel
    _session: requests.Session
    _client_version: Any

    class Config:
        arbitrary_types_allowed: bool

    def __init__(
        self,
        api_key: str,
        server_url: str = ...,
        verbose: VerboseLevel = ...,
    ) -> None: ...
    @property
    def server_url(self) -> str: ...
    @property
    def verbose(self) -> VerboseLevel: ...

class IndexClient(APIClient):
    api_key: Incomplete
    server_url: Incomplete
    verbose: Incomplete
    session: Incomplete

    def __init__(
        self,
        api_key: str,
        server_url: str | None = ...,
        verbose: VerboseLevel = ...,
    ) -> None: ...
    def files_index_create(
        self,
        name: str,
        read_images: bool = False,
        filepaths: list[Path] | list[str] | None = None,
        filesobjects: list[tuple[str, IO[bytes]]] | None = None,
        tags: list[str] | None = None,
    ) -> dict[str, Any] | None: ...
    def files_index_files_add(
        self,
        index_uuid: str,
        read_images: bool = False,
        filepaths: list[Path] | list[str] | None = None,
        filesobjects: list[tuple[str, IO[bytes]]] | None = None,
        tags: list[str] | None = None,
    ) -> dict[str, Any] | None: ...
    def files_index_files_delete(self, index_uuid: str, files_hashes: list[str]) -> dict[str, Any] | None: ...
    def files_index_delete(self, index_uuid: str) -> dict[str, Any] | None: ...
    def files_index_restore(self, index_uuid: str) -> dict[str, Any] | None: ...
    def files_index_rename(self, index_uuid: str, name: str) -> dict[str, Any] | None: ...
    def files_index_ask(
        self,
        index_uuid: str,
        question: str,
        output_language: str | None = None,
        active_files: list[str] | str | None = None,
        tags: list[str] | None = None,
    ) -> dict[str, Any] | None: ...
    def files_index_embed(self, index_uuid: str) -> dict[str, Any] | None: ...
    def files_index_details(self, index_uuid: str) -> dict[str, Any] | None: ...
    def files_index_list(self) -> dict[str, Any] | None: ...
    def files_index_tags_get(self: IndexClient, index_uuid: str) -> dict[str, Any] | None: ...
    def files_index_tags_update(self: IndexClient, index_uuid: str, tags: list[str]) -> dict[str, Any] | None: ...
    def files_index_files_tags_update(
        self: IndexClient,
        index_uuid: str,
        file_uuids: str | list[str],
        tags: list[str],
    ) -> dict[str, Any] | None: ...

class DelosClient(IndexClient):
    api_key: Incomplete
    server_url: Incomplete
    verbose: Incomplete
    session: Incomplete

    def __init__(
        self,
        api_key: str,
        server_url: str | None = ...,
        verbose: VerboseLevel = ...,
    ) -> None: ...
    def status_health(self) -> dict[str, Any] | None: ...
    def translate_text(
        self,
        text: str,
        output_language: str,
        input_language: str | None = None,
    ) -> dict[str, Any] | None: ...
    def translate_file(
        self,
        output_language: str,
        input_language: str | None = None,
        return_type: Literal["raw_text", "url", "file"] = "raw_text",
        filepath: Path | str | None = None,
        fileobject: tuple[str, IO[bytes]] | None = None,
    ) -> dict[str, Any] | None: ...
    def web_search(
        self,
        text: str,
        output_language: str | None = None,
        desired_urls: list[str] | str | None = None,
    ) -> dict[str, Any] | None: ...
    def llm_chat(
        self: DelosClient,
        text: str,
        model: str,
        messages: list[dict[str, str]] | None = None,
        temperature: float | None = None,
        response_format: ResponseFormat | str | None = None,
        **kwargs: Any,
    ) -> dict[str, Any] | None: ...
    async def llm_chat_stream(
        self: DelosClient,
        text: str,
        model: str,
        messages: list[dict[str, str]] | None = None,
        temperature: float | None = None,
        response_format: ResponseFormat | str | None = None,
        **kwargs: Any,
    ) -> AsyncGenerator[str | dict[str, Any], None]: ...
    async def llm_chat_beta(
        self: DelosClient,
        model: str,
        messages: list[dict[str, str]] | None = None,
        temperature: float | None = None,
        response_format: ResponseFormat | str | None = None,
        stream: bool = False,
        text: str | None = None,  # deprecated
        **kwargs: Any,
    ) -> dict[str, Any] | AsyncGenerator[str | dict[str, Any], None]: ...
    def llm_embed(self, text: str, model: str) -> dict[str, Any] | None: ...



"""Client for interacting with the Delos API."""

import json
import warnings
from collections.abc import AsyncGenerator
from pathlib import Path
from typing import IO, Annotated, Any, Callable, Literal, cast

import aiohttp
import requests
from pydantic import BaseModel, PrivateAttr

from .endpoints import Endpoints, RequestMethod
from .exceptions import APIKeyMissingError
from .models import ResponseFormat
from .releases import AllReleases
from .settings import VerboseLevel, delos_logger
from .utils import load_file, load_files

COSMOS_PLATFORM_BACKEND_URL = "https://platform.delos-suite.ai"


class APIClient(BaseModel):
    """Base class for API clients."""

    api_key: str
    _server_url: str = PrivateAttr()
    _verbose: VerboseLevel = PrivateAttr()
    _session: requests.Session = PrivateAttr()
    _client_version: Any = PrivateAttr()

    class Config:
        """Configuration for the API Clients."""

        arbitrary_types_allowed = True

    def __init__(
        self: "APIClient",
        api_key: str,
        server_url: str = COSMOS_PLATFORM_BACKEND_URL,
        verbose: VerboseLevel = VerboseLevel.INFO,
    ) -> None:
        """Initialize the client with the server URL and API key.

        Args:
            api_key: The API key to be used for requests.
            server_url: The URL of the server.
            verbose: The verbosity level of the client.
                0 - No logging
                1 - Only print the requests (default)
                2 - Print the requests and the response

        """
        super().__init__(api_key=api_key)
        if not api_key:
            raise APIKeyMissingError

        self._server_url = server_url
        self._verbose = verbose
        self._session = requests.Session()
        self._session.headers.update({"Authorization": f"Bearer {self.api_key}"})
        self._client_version = AllReleases[0]

        # Inner client logger
        self._logger_id = f"{__name__}.{id(self)}"
        self._logger = delos_logger.bind(client_id=self._logger_id)

        # Disable logger if verbose is NONE
        if self._verbose == VerboseLevel.NONE:
            delos_logger.disable(self._logger_id)

    @property
    def server_url(self) -> str:
        """Get the server URL."""
        return self._server_url

    @property
    def verbose(self) -> VerboseLevel:
        """Get the verbosity level."""
        return self._verbose

    def _log(self, message: str, level: int = VerboseLevel.INFO) -> None:
        """Log internal client messages based on verbosity setting.

        Args:
            message: The message to log.
            level: The verbosity level of the message.

        """
        if self._verbose >= level:
            if level == VerboseLevel.DEBUG:
                self._logger.debug(message)
            elif level == VerboseLevel.INFO:
                self._logger.info(message)

    def _build_request(
        self: "APIClient",
        endpoint: tuple[str, RequestMethod],
        data: dict[str, Any] | None = None,
        files: Any = None,
        params: dict[str, Any] | None = None,
        **kwargs: Any,
    ) -> tuple[str, Callable[..., requests.Response], dict[str, Any]]:
        """Prepare the request by setting up the URL, request function, and request arguments.

        Args:
            endpoint: A tuple containing the endpoint URL and the request type.
            data: The data to be sent in the request body (default is None).
            files: The files to be sent in the request (default is None).
            params: The query parameters to be sent in the request (default is None).
            kwargs: Specific keyword arguments to be passed to the request function.

        Returns:
            A tuple containing the URL, request function, and prepared request arguments.

        """
        url = f"{self.server_url}{self._client_version.suffix}{endpoint[0]}"
        request_method = endpoint[1]

        request_func = cast(
            "Callable[..., requests.Response] | None",
            {
                RequestMethod.GET: self._session.get,
                RequestMethod.POST: self._session.post,
                RequestMethod.PUT: self._session.put,
                RequestMethod.DELETE: self._session.delete,
            }.get(request_method),
        )

        if not request_func:
            unsupported_method_error = f"Unsupported HTTP method: {request_method}"
            self._log(unsupported_method_error, level=VerboseLevel.DEBUG)
            raise ValueError(unsupported_method_error)

        files_details = [name for (param, (name, content)) in files] if files else ""
        files_message = f"{len(files)} Files: {files_details}" if files else "No files"

        self._log(f"Request URL: {url}", VerboseLevel.DEBUG)
        self._log(f"Request Method: {request_method}", VerboseLevel.DEBUG)
        self._log(f"Request Data: {data}", VerboseLevel.DEBUG)
        self._log(f"Request Files: {files_message}", VerboseLevel.DEBUG)
        self._log(f"Request Params: {params}", VerboseLevel.DEBUG)

        form_data = {}
        if data:
            form_data.update({k: str(v) if v is not None else "" for k, v in data.items()})

        request_args = {
            "data": form_data,
            "files": files,
            "params": params,
            **kwargs,
        }

        return url, request_func, request_args

    def _build_async_request(
        self: "APIClient",
        endpoint: tuple[str, RequestMethod],
        data: dict[str, Any] | None = None,
        files: Any = None,
        params: dict[str, Any] | None = None,
        **kwargs: Any,
    ) -> tuple[str, Callable[..., Any], dict[str, Any]]:
        """Prepare the request by setting up the URL, request function, and request arguments.

        Args:
            endpoint: A tuple containing the endpoint URL and the request type.
            data: The data to be sent in the request body (default is None).
            files: The files to be sent in the request (default is None).
            params: The query parameters to be sent in the request (default is None).
            kwargs: Specific keyword arguments to be passed to the request function.

        Returns:
            A tuple containing the URL, request function, and prepared request arguments.

        """
        url = f"{self.server_url}{self._client_version.suffix}{endpoint[0]}"
        request_method = endpoint[1]

        request_func = getattr(aiohttp.ClientSession(), request_method.lower(), None)

        if not request_func:
            unsupported_method_error = f"Unsupported HTTP method: {request_method}"
            self._log(unsupported_method_error, level=VerboseLevel.DEBUG)
            raise ValueError(unsupported_method_error)

        files_details = [name for (param, (name, content)) in files] if files else ""
        files_message = f"{len(files_details)} Files: {files_details}" if files else "No files"

        self._log(f"Request URL: {url}", VerboseLevel.DEBUG)
        self._log(f"Request Method: {request_method}", VerboseLevel.DEBUG)
        self._log(f"Request Data: {data}", VerboseLevel.DEBUG)
        self._log(f"Request Files: {files_message}", VerboseLevel.DEBUG)
        self._log(f"Request Params: {params}", VerboseLevel.DEBUG)

        headers = {"Authorization": f"Bearer {self.api_key}"}

        request_args = {
            "headers": headers,
            "data": data,
            "params": params,
            **kwargs,
        }

        if files:
            form_data = aiohttp.FormData()
            for param, (name, content) in files:
                form_data.add_field(param, content, filename=name)
            request_args["data"] = form_data

        return url, request_func, request_args

    def _make_request(
        self: "APIClient",
        endpoint: tuple[str, RequestMethod],
        data: dict[str, Any] | None = None,
        files: Any = None,
        params: dict[str, Any] | None = None,
        **kwargs: Any,
    ) -> dict[str, Any] | None:
        """Make a request to the specified endpoint with the given data and files.

        Args:
            api_key: The Delos key to be used for the request.
            endpoint: A tuple containing the endpoint URL and the request type.
            data: The data to be sent in the request body (default is None).
            files: The files to be sent in the request (default is None).
            params: The query parameters to be sent in the request (default is None).
            kwargs: Specific keyword arguments to be passed to the request function.

        Returns:
            The response from the request as a dictionary, or None if an error occurred.

        """
        url, request_func, request_args = self._build_request(endpoint, data, files, params, **kwargs)

        try:
            response = request_func(url, **request_args)
            response.raise_for_status()

            if self.verbose >= VerboseLevel.INFO:
                self._log(f"Response: {response.json()}", VerboseLevel.INFO)
            return response.json()

        except requests.exceptions.RequestException:
            error_message = f"Request to {url} failed"
            self._logger.exception(error_message)
            raise

    async def _make_streaming_request(
        self: "APIClient",
        endpoint: tuple[str, RequestMethod],
        data: dict[str, Any] | None = None,
        files: Any = None,
        params: dict[str, Any] | None = None,
        **kwargs: Any,
    ) -> AsyncGenerator[str | dict[str, Any], None]:
        """Make a request to the specified streaming endpoint with the given data and files.

        Args:
            api_key: The Delos key linked to this client.
            endpoint: A tuple containing the endpoint URL and the request type.
            data: The data to be sent in the request body (default is None).
            files: The files to be sent in the request (default is None).
            params: The query parameters to be sent in the request (default is None).
            kwargs: Specific keyword arguments to be passed to the request function.

        Returns:
            The streaming response from the request.

        """
        if endpoint[1] != RequestMethod.POST:
            streaming_error_message = "Streaming is only supported for POST requests."
            self._logger.warning(streaming_error_message)
            raise ValueError(streaming_error_message)

        url, _request_func, request_args = self._build_async_request(endpoint, data, files, params, **kwargs)

        session = aiohttp.ClientSession()
        try:
            # Create the session separately so we can ensure it's closed
            async with session.post(url, **request_args) as response:
                response.raise_for_status()
                async for line in response.content.iter_any():
                    decoded_line = line.decode("utf-8").strip()
                    if decoded_line:
                        yield decoded_line

            if self._verbose >= VerboseLevel.INFO:
                self._log("Streaming completed", VerboseLevel.INFO)

        except aiohttp.ClientError:
            error_message = f"Request to {url} (streaming) failed"
            self._logger.exception(error_message)
            raise

        finally:
            await session.close()
            if self._verbose >= VerboseLevel.INFO:
                self._log("Session closed", VerboseLevel.INFO)


class IndexClient(APIClient):
    """Client for interacting with the Delos Index API.

    Attributes:
        server_url: The URL of the server.
        api_key: The API key to be used for requests.

    """

    def __init__(
        self,
        api_key: str,
        server_url: str = COSMOS_PLATFORM_BACKEND_URL,
        verbose: VerboseLevel = VerboseLevel.INFO,
    ) -> None:
        """Initialize the DelosClient.

        Args:
            api_key: The Delos key linked to this client.
            server_url: The URL of the server.
            verbose: The verbosity level of the client.

        """
        super().__init__(api_key=api_key, server_url=server_url, verbose=verbose)

    def files_index_create(
        self: "IndexClient",
        name: str,
        read_images: bool = False,
        filepaths: list[Path] | list[str] | None = None,
        fileobjects: list[tuple[str, IO[bytes]]] | None = None,
        filesobjects: Annotated[
            list[tuple[str, IO[bytes]]] | None,
            "Deprecated: Use `fileobjects` (without intermediate S) instead",
        ] = None,
        tags: list[str] | None = None,
    ) -> dict[str, Any] | None:
        """Make a request to create an index.

        Args:
            name: The name of the index.
            read_images: Whether to read images from the files (Optional). Default is False.

            filepaths: A list of file paths to be indexed. Provide either `filepaths` or `fileobjects`, not both.
            fileobjects: A list of file objects to be indexed.
            filesobjects: Old name ("fileSobjects", new is "fileobjects") - SOON TO BE DEPRECATED.
            tags: A list of tags to be enabled for the index.

        Returns:
            The server response.

        """
        if filesobjects:
            deprecation_warning = (
                "The parameter `filesobjects` has been renamed. Old parameter name soon will be deprecated. "
                "Use parameter `fileobjects` (instead of `fileSobjects`)'"
            )
            self._logger.warning(deprecation_warning)
            warnings.warn(
                deprecation_warning,
                DeprecationWarning,
                stacklevel=2,
            )
            fileobjects = filesobjects

        if (filepaths is None and fileobjects is None) or (filepaths is not None and fileobjects is not None):
            warning_message = "Provide only `filepaths` or `fileobjects`, not both or neither."
            raise ValueError(warning_message)

        return self._make_request(
            Endpoints.FILES.INDEX_CREATE.value,
            data={"name": name, "read_images": read_images, "tags": tags},
            files=load_files(filepaths, fileobjects),
        )

    def files_index_files_add(
        self: "IndexClient",
        index_uuid: str,
        read_images: bool = False,
        filepaths: list[Path] | list[str] | None = None,
        fileobjects: list[tuple[str, IO[bytes]]] | None = None,
        filesobjects: Annotated[
            list[tuple[str, IO[bytes]]] | None,
            "Deprecated: Use `fileobjects` (without intermediate S) instead",
        ] = None,
        tags: list[str] | None = None,
    ) -> dict[str, Any] | None:
        """Make a request to add files to an index.

        Args:
            index_uuid: The index UUID.
            read_images: Whether to read images from the files (Optional). Default is False.

            filepaths: A list of file paths to be added to the index.
            fileobjects: A list of file objects to be indexed. Provide either `filepaths` or `fileobjects`, not both.
            filesobjects: Old name ("fileSobjects", new is "fileobjects") - SOON TO BE DEPRECATED.
            tags: A list of tags to be applied to ALL FILES in this operation.

        Returns:
            The server response.

        """
        if filesobjects:
            deprecation_warning = (
                "The parameter `filesobjects` has been renamed. Old parameter name soon will be deprecated. "
                "Use parameter `fileobjects` (instead of `fileSobjects`)'"
            )
            self._logger.warning(deprecation_warning)
            warnings.warn(
                deprecation_warning,
                DeprecationWarning,
                stacklevel=2,
            )
            fileobjects = filesobjects

        if (filepaths is None and fileobjects is None) or (filepaths is not None and fileobjects is not None):
            warning_message = "Provide only `filepaths` or `fileobjects`, not both or neither."
            raise ValueError(warning_message)

        return self._make_request(
            Endpoints.FILES.INDEX_ADD_FILES.value,
            data={"index_uuid": index_uuid, "read_images": read_images, "tags": tags},
            files=load_files(filepaths, fileobjects),
        )

    def files_index_files_delete(
        self: "IndexClient",
        index_uuid: str,
        files_hashes: list[str],
    ) -> dict[str, Any] | None:
        """Make a request to delete files from an index.

        Args:
            index_uuid: The index UUID.
            files_hashes: A list of file hashes to be deleted from the index.

        Returns:
            The server response.

        """
        debug_message = (
            f"{len(files_hashes)} Files to be deleted: {files_hashes} (type {type(files_hashes)} "
            f"- first item:{files_hashes[0]} ({type(files_hashes[0])}))"
        )
        self._logger.warning(debug_message)
        files_hashes_str = [str(file_hash) for file_hash in files_hashes]
        data = {"index_uuid": index_uuid, "files_hashes": files_hashes_str}
        return self._make_request(Endpoints.FILES.INDEX_DELETE_FILES.value, data=data)

    def files_index_delete(self: "IndexClient", index_uuid: str) -> dict[str, Any] | None:
        """Make a request to delete an index.

        Args:
            index_uuid: The index UUID.

        Returns:
            The server response.

        """
        data = {"index_uuid": index_uuid}
        return self._make_request(Endpoints.FILES.INDEX_DELETE.value, data=data)

    def files_index_restore(self: "IndexClient", index_uuid: str) -> dict[str, Any] | None:
        """Make a request to restore an index.

        Args:
            index_uuid: The index UUID.

        Returns:
            The server response.

        """
        data = {"index_uuid": index_uuid}
        return self._make_request(Endpoints.FILES.INDEX_RESTORE.value, data=data)

    def files_index_rename(
        self: "IndexClient",
        index_uuid: str,
        name: str,
    ) -> dict[str, Any] | None:
        """Make a request to rename an index.

        Args:
            index_uuid: The index UUID.
            name: The new name for the index.

        Returns:
            The server response.

        """
        data = {"index_uuid": index_uuid, "name": name}
        return self._make_request(Endpoints.FILES.INDEX_RENAME.value, data=data)

    def files_index_ask(
        self: "IndexClient",
        index_uuid: str,
        question: str,
        output_language: str | None = None,
        active_files: list[str] | str | None = None,
        tags: list[str] | None = None,
    ) -> dict[str, Any] | None:
        """Make a request to ask a question about the index contents.

        Args:
            index_uuid: The index UUID.
            question: The question to be asked.
            output_language: The output language for the question.
            active_files: The file_id of the files to be used for the question.
            tags: Subset of files to be used in this research (when not provided, all tags are enabled).

        Returns:
            The server response.

        """
        data = {
            "index_uuid": str(index_uuid),
            "question": question,
            "active_files": json.dumps(active_files) if isinstance(active_files, list) else active_files,
        }
        if output_language:
            data["output_language"] = output_language
        if tags:
            data["tags"] = json.dumps(tags) if isinstance(tags, list) else tags

        return self._make_request(Endpoints.FILES.INDEX_ASK.value, data=data)

    def files_index_embed(self: "IndexClient", index_uuid: str) -> dict[str, Any] | None:
        """Make a request to embed an index.

        Args:
            index_uuid: The index UUID.

        Returns:
            The server response.

        """
        data = {"index_uuid": index_uuid}
        return self._make_request(Endpoints.FILES.INDEX_EMBED.value, data=data)

    def files_index_details(self: "IndexClient", index_uuid: str) -> dict[str, Any] | None:
        """Make a request to get details of an index.

        Args:
            index_uuid: The index UUID.

        Returns:
            The server response.

        """
        params = {"index_uuid": index_uuid}
        return self._make_request(Endpoints.FILES.INDEX_DETAILS.value, params=params)

    def files_index_list(self: "IndexClient") -> dict[str, Any] | None:
        """Make a request to list all indexes."""
        return self._make_request(Endpoints.FILES.INDEX_LIST.value)

    def files_index_tags_get(self: "IndexClient", index_uuid: str) -> dict[str, Any] | None:
        """Make a request to get the tags of a given index."""
        params = {"index_uuid": index_uuid}
        return self._make_request(Endpoints.FILES_TAGS.INDEX_GET_TAGS.value, params=params)

    def files_index_tags_update(
        self: "IndexClient",
        index_uuid: str,
        tags: list[str],
    ) -> dict[str, Any] | None:
        """Make a request to update the tags of a given index."""
        data = {"index_uuid": index_uuid, "tags": tags}
        return self._make_request(Endpoints.FILES_TAGS.INDEX_UPDATE_TAGS.value, data=data)

    def files_index_files_tags_update(
        self: "IndexClient",
        index_uuid: str,
        files_uuids: str | list[str],
        tags: list[str],
    ) -> dict[str, Any] | None:
        """Make a request to update the tags of a given file."""
        data = {"index_uuid": index_uuid, "files_ids": files_uuids, "tags": tags}
        return self._make_request(Endpoints.FILES_TAGS.INDEX_UPDATE_FILE_TAGS.value, data=data)


class DelosClient(IndexClient):
    """Client for interacting with the Delos API.

    Attributes:
        server_url: The URL of the server.
        api_key: The API key to be used for requests.

    """

    def __init__(
        self,
        api_key: str,
        server_url: str = COSMOS_PLATFORM_BACKEND_URL,
        verbose: VerboseLevel = VerboseLevel.INFO,
    ) -> None:
        """Initialize the DelosClient.

        Args:
            api_key: The Delos key linked to this client.
            server_url: The URL of the server.
            verbose: The verbosity level of the client.

        """
        super().__init__(api_key=api_key, server_url=server_url, verbose=verbose)

    def status_health(
        self: "DelosClient",
    ) -> dict[str, Any] | None:
        """Make a request to check the health of the server."""
        return self._make_request(Endpoints.STATUS.HEALTH.value)

    def translate_text(
        self: "DelosClient",
        text: str,
        output_language: str,
        input_language: str | None = None,
    ) -> dict[str, Any] | None:
        """Make a request to translate text.

        Args:
            text: The text to be translated.
            output_language: The output language for the translation.
            input_language: The input language for the translation (Optional).

        Returns:
            The server response.

        """
        data = {
            "text": text,
            "output_language": output_language,
        }
        if input_language:
            data["input_language"] = input_language
        return self._make_request(endpoint=Endpoints.TRANSLATE.TRANSLATE_TEXT.value, data=data)

    def translate_file(
        self: "DelosClient",
        output_language: str,
        input_language: str | None = None,
        return_type: Literal["raw_text", "url", "file"] = "raw_text",
        filepath: Path | str | None = None,
        fileobject: tuple[str, IO[bytes]] | None = None,
    ) -> dict[str, Any] | None:
        """Make a request to translate a file.

        Args:
            output_language: The output language for the translation.
            input_language: The input language for the translation (Optional).
            return_type: The type of return for the translation (Optional). Default is "raw_text".

            filepath: The file path to be translated.
            fileobject: The file object to be translated. Provide either filepath or fileobject, not both.

        Returns:
            The server response.

        """
        files = load_file(filepath, fileobject)
        data = {
            "output_language": output_language,
            "return_type": return_type,
        }
        if input_language:
            data["input_language"] = input_language

        return self._make_request(Endpoints.TRANSLATE.TRANSLATE_FILE.value, data=data, files=files)

    def web_search(
        self: "DelosClient",
        text: str,
        output_language: str | None = None,
        desired_urls: list[str] | str | None = None,
    ) -> dict[str, Any] | None:
        """Make a request to perform a search.

        Args:
            text: The text to be searched.
            output_language: The output language for the search (Optional).
            desired_urls: The desired URLs to be priviledged in the search (Optional).

        Returns:
            The server response.

        """
        data = {"text": text}
        if output_language:
            data["output_language"] = output_language
        if desired_urls:
            data["desired_urls"] = str(desired_urls)

        return self._make_request(Endpoints.WEB.SEARCH.value, data)

    async def llm_chat_beta(
        self: "DelosClient",
        model: str,
        messages: list[dict[str, str]] | None = None,
        temperature: float | None = None,
        response_format: ResponseFormat | str | None = None,
        stream: bool = False,
        text: str | None = None,  # deprecated
        **kwargs: Any,
    ) -> dict[str, Any] | AsyncGenerator[str | dict[str, Any], None] | None:
        """Make a request to chat with the LLM.

        Args:
            text: The text to be chatted.
            messages: The history of the conversation (Optional. Default is None).
            model: The model to be used for chatting.
            temperature: The temperature for the chat (Optional. Default is 0.7).
            response_format: The response format for the chat (Optional).
                For example: `response_format = {"type":"json_object"}`
            stream: Whether to stream the response or not.
            kwargs: Other specific keyword arguments to be passed to the request function.

        Returns:
            The server response.

        """
        if messages is None and text is None:
            missing_message = "Either `messages` or `text` must be provided. Note that `text` will be deprecated soon."
            raise ValueError(missing_message)

        messages_list = messages or []
        deprecated_text_message = (
            "The parameter `text:str`, soon will be deprecated. "
            "Send your message in the list of `messages:list[dict[str,str]]`, following the message format:"
            "`message: dict[str,str] = {'role':'user', 'content':text_content}`."
        )

        if not messages:
            self._logger.warning("List of `messages` will soon be required (`text` parameter deprecated).")

        if text or not messages:
            self._logger.warning(deprecated_text_message)
            warnings.warn(
                deprecated_text_message,
                DeprecationWarning,
                stacklevel=2,
            )
            if text is not None:
                messages_list.append({"role": "user", "content": text})

        data = {"model": model, "stream": stream}
        data["messages"] = str(messages_list)

        if response_format is not None:
            if isinstance(response_format, str):
                warning_message = (
                    "The parameter `response_format:str`, soon will be deprecated. "
                    "Use instead `response_format:dict[str, Any]` which must contain a `type` key. For example: "
                    '`response_format={"type":"json_object"}` or `response_format={"type":"text"}`.'
                )
                self._logger.warning(warning_message)
                warnings.warn(
                    warning_message,
                    DeprecationWarning,
                    stacklevel=2,
                )
                data["response_format"] = response_format

            else:
                parsed_format = json.dumps(response_format)
                data["response_format"] = parsed_format

        if temperature is not None:
            data["temperature"] = str(temperature)

        data.update(kwargs)

        if stream:
            return self._make_streaming_request(endpoint=Endpoints.LLM.CHAT_BETA.value, data=data)
        return self._make_request(endpoint=Endpoints.LLM.CHAT_BETA.value, data=data)

    def llm_chat(
        self: "DelosClient",
        text: str,
        model: str,
        messages: list[dict[str, str]] | None = None,
        temperature: float | None = None,
        response_format: ResponseFormat | str | None = None,  # str: deprecated
        **kwargs: Any,
    ) -> dict[str, Any] | None:
        """Make a request to chat with the LLM.

        Args:
            text: The text to be chatted.
            messages: The history of the conversation (Optional. Default is None).
            model: The model to be used for chatting.
            temperature: The temperature for the chat (Optional. Default is 0.7).
            response_format: The response format for the chat (Optional).
                For example: `response_format = {"type":"json_object"}`
            kwargs: Other specific keyword arguments to be passed to the request function.

        Returns:
            The server response.

        """
        data = {"text": text, "model": model}
        if messages is not None:
            data["messages"] = str(messages)
        if response_format is not None:
            if isinstance(response_format, str):
                warning_message = (
                    "The parameter `response_format:str`, soon will be deprecated. "
                    "Use instead `response_format:dict[str, Any]` which must contain a `type` key. For example: "
                    '`response_format={"type":"json_object"}` or `response_format={"type":"text"}`.'
                )
                self._logger.warning(warning_message)
                warnings.warn(
                    warning_message,
                    DeprecationWarning,
                    stacklevel=2,
                )
                data["response_format"] = response_format

            else:
                parsed_format = json.dumps(response_format)
                data["response_format"] = parsed_format

        if temperature is not None:
            data["temperature"] = str(temperature)

        data.update(kwargs)
        return self._make_request(endpoint=Endpoints.LLM.CHAT.value, data=data)

    async def llm_chat_stream(
        self: "DelosClient",
        text: str,
        model: str,
        messages: list[dict[str, str]] | None = None,
        temperature: float | None = None,
        response_format: ResponseFormat | str | None = None,  # str: deprecated
        **kwargs: Any,
    ) -> AsyncGenerator[str | dict[str, Any], None]:
        """Make a streaming request to chat with the LLM.

        Args:
            text: The text to be chatted.
            model: The model to be used for chatting.
            messages: The history of the conversation (Optional. Default is None).
            temperature: The temperature for the chat (Optional. Default is 0.7).
            response_format: The response format for the chat (Optional).
                For example: `response_format = {"type":"json_object"}`
            kwargs: Other specific keyword arguments to be passed to the request function.

        Returns:
            The server response.

        """
        data = {"text": text, "model": model}
        if messages is not None:
            data["messages"] = str(messages)

        if response_format is not None:
            if isinstance(response_format, str):
                self._logger.warning(
                    "The parameter `response_format:str`, soon will be deprecated. "
                    "Use instead `response_format:dict[str, Any]` which must contain a `type` key. For example: "
                    '`response_format={"type":"json_object"}` or `response_format={"type":"text"}`.',
                )
                data["response_format"] = response_format
            else:
                data["response_format"] = str(response_format)

        if temperature is not None:
            data["temperature"] = str(temperature)

        data.update(kwargs)

        stream = self._make_streaming_request(endpoint=Endpoints.LLM.CHAT_STREAM.value, data=data)
        async for chunk in stream:
            yield chunk

    def llm_embed(
        self: "DelosClient",
        text: str,
        model: str,
    ) -> dict[str, Any] | None:
        """Make a request to embed data using the LLM.

        Args:
            text: The text to be embedded.
            model: The model to use for embedding.

        Returns:
            The server response.

        """
        return self._make_request(Endpoints.LLM.EMBED.value, data={"text": text, "model": model})



"""Utility functions for the Delos API client."""

import json
from collections.abc import Generator
from contextlib import suppress
from pathlib import Path
from typing import IO, Any

import requests

from .exceptions import InvalidInputError


def read_streaming_response(response: requests.Response) -> Generator[dict[str, Any], None, None]:
    """Process the streaming response and yield parsed data."""
    for line in response.iter_lines():
        if not line:
            continue

        line_read = line.decode("utf-8")
        if line_read == "data: [DONE]":
            break

        if line_read.startswith("data: "):
            data = line_read[6:]
            with suppress(json.JSONDecodeError):
                yield json.loads(data)

        elif line_read.startswith("0:"):
            yield line_read[2:].strip('"')


def process_streaming_response(response: requests.Response) -> Generator[str, None, None]:
    """Process the streaming response and yield raw data lines."""
    for line in response.iter_lines():
        if line:
            yield line.decode("utf-8")


def _check_file(filepath: Path | str | None, fileobject: tuple[str, IO[bytes]] | None) -> None:
    """Check if filepath and / or fileobject are provided."""
    if not filepath and not fileobject:
        error_message = "Either filepath or fileobject must be provided."
        raise InvalidInputError(error_message)

    if filepath and fileobject:
        error_message = "Provide either filepath or fileobject, not both."
        raise InvalidInputError(error_message)


def _check_files(
    filepaths: list[Path] | list[str] | None,
    filesobjects: list[tuple[str, IO[bytes]]] | None = None,
) -> None:
    """Check if filepath and / or fileobject are provided."""
    if not filepaths and not filesobjects:
        error_message = "Either filepaths or filesobjects must be provided."
        raise InvalidInputError(error_message)

    if filepaths and filesobjects:
        error_message = "Provide either filepaths or filesobjects, not both."
        raise InvalidInputError(error_message)


def load_file(
    filepath: Path | str | None = None,
    fileobject: tuple[str, IO[bytes]] | None = None,
) -> list[tuple[str, Any]]:
    """Load a file content."""
    _check_file(filepath, fileobject)
    if filepath:
        path = Path(filepath) if isinstance(filepath, str) else filepath
        with path.open("rb") as file:
            return [("file", (path.name, file.read()))]

    elif fileobject:
        filename, file_object = fileobject
        return [("file", (filename, file_object.read()))]
    return []


def load_files(
    filepaths: list[Path] | list[str] | None = None,
    fileobjects: list[tuple[str, IO[bytes]]] | None = None,
) -> list[tuple[str, Any]]:
    """Load files contents."""
    _check_files(filepaths, fileobjects)

    files = []
    if filepaths:
        for fp in filepaths:
            path = Path(fp) if isinstance(fp, str) else fp
            with path.open("rb") as file:
                files.append(("files", (path.name, file.read())))

    elif fileobjects:
        for filename, file_object in fileobjects:
            files.append(("files", (filename, file_object.read())))
    return files



class APIKeyMissingError(Exception):
    def __init__(self) -> None: ...

class ResponseFormatTypeMissingError(Exception):
    def __init__(self) -> None: ...

class InvalidInputError(Exception):
    def __init__(self, message: str) -> None: ...



from typing import TypedDict

from typing_extensions import TypeAlias

class JSONSchema(TypedDict):
    name: str
    description: str | None
    schema_: dict[str, object] | None
    strict: bool | None

class ResponseFormatJSONSchema(TypedDict):
    json_schema: JSONSchema
    type: str

class ResponseFormatJSONObject(TypedDict):
    type: str

class ResponseFormatText(TypedDict):
    type: str

ResponseFormat: TypeAlias = ResponseFormatText | ResponseFormatJSONObject | ResponseFormatJSONSchema



"""Delos client."""

from .client import DelosClient
from .endpoints import DelosEndpoints, Endpoints, FileEndpoints
from .settings import VerboseLevel, delos_logger
from .utils import process_streaming_response, read_streaming_response

__all__ = [
    "DelosClient",
    "DelosEndpoints",
    "Endpoints",
    "FileEndpoints",
    "VerboseLevel",
    "delos_logger",
    "process_streaming_response",
    "read_streaming_response",
]



from datetime import datetime
from enum import Enum
from typing import Any

from _typeshed import Incomplete
from pydantic import BaseModel

from .endpoints import DelosEndpoints as DelosEndpoints
from .endpoints import Endpoints as Endpoints

class Release(Enum):
    V1 = "v1"

class DelosClientReleases(BaseModel):
    version: Release
    release_date: datetime
    details: dict[str, Any]
    suffix: str
    available_endpoints: list[DelosEndpoints]

first_release: Incomplete
AllReleases: list[DelosClientReleases]



"""Definition of Delos endpoints and their request types."""

from enum import Enum


class RequestMethod(str, Enum):
    """Enum representing the type of HTTP requests."""

    GET = "GET"
    POST = "POST"
    PUT = "PUT"
    DELETE = "DELETE"


class StatusEndpoints(Enum):
    """Enum containing the status service endpoints and their request types."""

    HEALTH = ("/health", RequestMethod.GET)


class TranslateEndpoints(Enum):
    """Enum containing the translate service endpoints and their request types."""

    TRANSLATE_TEXT = ("/translate-text", RequestMethod.POST)
    TRANSLATE_FILE = ("/translate-file", RequestMethod.POST)


class WebEndpoints(Enum):
    """Enum containing the web service endpoints and their request types."""

    SEARCH = ("/search", RequestMethod.POST)


class FileEndpoints(Enum):
    """Enum containing the file service endpoints and their request types."""

    PARSER = ("/files/parse", RequestMethod.POST)
    INDEX_CREATE = ("/files/create_index_and_parse", RequestMethod.POST)
    INDEX_ADD_FILES = ("/files/add_files_to_index", RequestMethod.POST)
    INDEX_RESTORE = ("/files/restore_index", RequestMethod.PUT)
    INDEX_RENAME = ("/files/rename_index", RequestMethod.PUT)
    INDEX_DELETE_FILES = ("/files/delete_files_from_index", RequestMethod.DELETE)
    INDEX_DELETE = ("/files/delete_index", RequestMethod.DELETE)

    INDEX_ASK = ("/files/ask_index", RequestMethod.POST)
    INDEX_EMBED = ("/files/embed_index", RequestMethod.POST)
    INDEX_LIST = ("/files/list_index", RequestMethod.GET)
    INDEX_DETAILS = ("/files/index_details", RequestMethod.GET)


class FileTagsEndpoints(Enum):
    """Enum containing the file tags service endpoints and their request types."""

    INDEX_GET_TAGS = ("/files/get_index_tags", RequestMethod.GET)
    INDEX_UPDATE_TAGS = ("/files/update_index_tags", RequestMethod.PUT)
    INDEX_UPDATE_FILE_TAGS = ("/files/update_index_files_tags", RequestMethod.PUT)


class LlmEndpoints(Enum):
    """Enum containing the LLM service endpoints and their request types."""

    CHAT = ("/llm/chat", RequestMethod.POST)
    CHAT_STREAM = ("/llm/chat_stream", RequestMethod.POST)
    CHAT_BETA = ("/llm/chat/beta", RequestMethod.POST)
    EMBED = ("/llm/embed", RequestMethod.POST)


class Endpoints:
    """Class grouping all the different types of endpoints."""

    STATUS = StatusEndpoints
    WEB = WebEndpoints
    TRANSLATE = TranslateEndpoints
    FILES = FileEndpoints
    FILES_TAGS = FileTagsEndpoints
    LLM = LlmEndpoints


DelosEndpoints = StatusEndpoints | WebEndpoints | TranslateEndpoints | FileEndpoints | FileTagsEndpoints | LlmEndpoints



from enum import Enum

import typing_extensions

class RequestMethod(str, Enum):
    GET = "GET"
    POST = "POST"
    PUT = "PUT"
    DELETE = "DELETE"

class StatusEndpoints(Enum):
    HEALTH = ...

class TranslateEndpoints(Enum):
    TRANSLATE_TEXT = ...
    TRANSLATE_FILE = ...

class WebEndpoints(Enum):
    SEARCH = ...

class FileEndpoints(Enum):
    PARSER = ...
    INDEX_CREATE = ...
    INDEX_ADD_FILES = ...
    INDEX_RESTORE = ...
    INDEX_RENAME = ...
    INDEX_DELETE_FILES = ...
    INDEX_DELETE = ...
    INDEX_ASK = ...
    INDEX_EMBED = ...
    INDEX_LIST = ...
    INDEX_DETAILS = ...

class FileTagsEndpoints(Enum):
    INDEX_GET_TAGS = ...
    INDEX_UPDATE_TAGS = ...
    INDEX_UPDATE_FILE_TAGS = ...

class LlmEndpoints(Enum):
    CHAT = ...
    CHAT_STREAM = ...
    CHAT_BETA = ...
    EMBED = ...

class Endpoints:
    STATUS = StatusEndpoints
    WEB = WebEndpoints
    TRANSLATE = TranslateEndpoints
    FILES = FileEndpoints
    FILES_TAGS = FileTagsEndpoints
    LLM = LlmEndpoints

DelosEndpoints: typing_extensions.TypeAlias = (
    StatusEndpoints | WebEndpoints | TranslateEndpoints | FileEndpoints | FileTagsEndpoints | LlmEndpoints
)



"""Models for the Delos client."""

from typing import Literal, Required, TypedDict

from typing_extensions import TypeAlias


class JSONSchema(TypedDict):
    """JSON Schema for the response format.

    Attributes:
        name: The name of the response format.
            Must be a-z, A-Z, 0-9, or contain underscores and dashes, with a maximum length of 64 characters.

        description: A description of what the response format is for,
            used by the model to determine how to respond in the format.

        schema: The schema for the response format, described as a JSON Schema object.

        strict: Whether to enable strict schema adherence when generating the output.
            If set to true, the model will always follow the exact schema defined in the `schema` field.
            Only a subset of JSON Schema is supported when `strict` is `true`.

        type: The type of response format being defined: `json_schema`

    """

    name: str
    description: str | None
    schema_: dict[str, object] | None
    strict: bool | None


class ResponseFormatJSONSchema(TypedDict):
    """Response format through a JSON Schema.

    Attributes:
        type: The type of response format being defined: `json_schema`

    """

    json_schema: JSONSchema
    type: Literal["json_schema"]


class ResponseFormatJSONObject(TypedDict):
    """Response format through a JSON Object.

    Attributes:
        type: The type of response format being defined: `json_object`

    """

    type: Required[Literal["json_object"]]


class ResponseFormatText(TypedDict):
    """Response format as text.

    Attributes:
        type: The type of response format being defined: `text`

    """

    type: Required[Literal["text"]]


ResponseFormat: TypeAlias = ResponseFormatText | ResponseFormatJSONObject | ResponseFormatJSONSchema



from collections.abc import Generator
from pathlib import Path
from typing import IO, Any

import requests

def read_streaming_response(response: requests.Response) -> Generator[dict[str, Any], None, None]: ...
def process_streaming_response(response: requests.Response) -> Generator[dict[str, Any], None, None]: ...
def load_file(
    filepath: Path | str | None = None,
    fileobject: tuple[str, IO[bytes]] | None = None,
) -> list[tuple[str, Any]]: ...
def load_files(
    filepaths: list[Path] | list[str] | None = None,
    fileobjects: list[tuple[str, IO[bytes]]] | None = None,
) -> list[tuple[str, Any]]: ...



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['delos']

package_data = \
{'': ['*']}

modules = \
['py']
install_requires = \
['aiofiles>=24.1.0,<25.0.0',
 'aiohttp>=3.11.13,<4.0.0',
 'fastapi>=0.115.5,<0.116.0',
 'loguru>=0.7.2,<0.8.0',
 'pydantic>=2.10.5,<3.0.0',
 'requests>=2.32.3,<3.0.0']

setup_kwargs = {
    'name': 'delos',
    'version': '0.3.9.1',
    'description': 'Delos client.',
    'long_description': '# Delos Delos\n\n## Delos client for interacting with the Delos API.\n\n# Installation\n\nTo install the package, use `poetry`:\n\n```bash\npoetry add delos\n```\n\nOr if you use the default `pip`:\n\n```bash\npip install delos\n```\n\n# Client Initialization\n\nYou can create an **API key** to access all services through the **Dashboard** in **DelosPlatform**\n`https://platform.delos-suite.ai`.\n\n![API Key creation in Delos Platform](https://i.ibb.co/6mvm1hQ/api-key-create.png)\n\nTo create a `Delos` client instance, you need to initialize it with your API key:\n\n```python\nfrom delos import DelosClient\n\nclient = DelosClient(api_key="your-delos-api-key")\n\n```\n\n# Endpoints\n\nThis `delos` client provides access to the following endpoints:\n\n**Status Endpoints**\n\n- `status_health`: Check the health of the server.\n\n**Translate Endpoints**\n\n- `translate_text`: Translate text.\n- `translate_file`: Translate a file.\n\n**Web Endpoints**\n\n- `web_search`: Perform a web search.\n\n**LLM Endpoints**\n\n- `chat`: Chat with the LLM.\n- `embed`: Embed data into the LLM.\n\n**Files Endpoints**\n\nA **single file** can be read and parsed with the universal parser endpoint:\n\n- `files_parse`: Parse a file to extract the pages, chunks or subchunks.\n\nAn **index** groups a set of files in order to be able to query them using natural language. There are several\noperations regarding **index management**:\n\n- `files_index_create`: Create an index.\n- `files_index_files_add`: Add files to an index.\n- `files_index_files_delete`: Delete files from an index.\n- `files_index_delete`: Delete an index.\n- `files_index_restore`: Restore a deleted index.\n- `files_index_rename`: Rename an index.\n\nAnd regarding **index querying**\n\n- `files_index_ask`: Ask a question about the index documents (it requires that your `index.status.vectorized` is set to\n  `True`).\n- `files_index_embed`: Embed or vectorize the index contents.\n- `files_index_list`: List all indexes.\n- `files_index_details`: Get details of an index.\n\nThese endpoints are accessible through `delos` client methods.\n\n> ℹ️ **Info:** For all the **endpoints**, there are specific **parameters** that are required regarding the data to be\n> sent to the API.\n>\n> Endpoints may expect `text` or `files` to operate with, the `output_language` for your result, the `index_uuid` that\n> identifies the set of documents, the `model` to use for the LLM operations, etc.\n>\n> You can find the standardized parameters like the `return_type` for file translation and the `extract_type` for file\n> parser in the appropiate endpoint.\n\n---\n\n## Status Endpoints\n\n### Status Health Request\n\nTo **check the health** of the server and the validity of your API key:\n\n```python\nresponse = client.status_health()\nif response:\n    print(f"Response: {response}")\n```\n\n---\n\n## Translate Endpoints\n\n### 1. Translate Text Request\n\nTo **translate text**, you can use the `translate_text` method:\n\n```python\nresponse = client.translate_text(\n                        text="Hello, world!",\n                        output_language="fr"\n                    )\nif response:\n    print(f"Translated Text: {response}")\n```\n\n### 2. Translate File Request\n\nTo **translate a file**, use the `translate_file` method:\n\n```python\nlocal_filepath_1 = Path("/path/to/file1.pdf")\n\nresponse = client.translate_file(\n                        filepath=local_filepath_1,\n                        output_language="fr",\n                    )\n```\n\nAccording to the type of file translation you prefer, you can choose the `return_type` parameter to:\n\n| return_type        |                                                     |\n| ------------------ | --------------------------------------------------- |\n| raw_text `Default` | Returns the translated text only                    |\n| url                | Return the translated file with its layout as a URL |\n\n> 💡 **Tip:** For faster and economical translations, set the `return_type` to `raw_text` to request to translate only\n> the **text content**, without the file layout.\n\n```python\nlocal_filepath_1 = Path("/path/to/file1.pdf")\nlocal_filepath_2 = Path("/path/to/file2.pdf")\n\n# Set return_type=\'raw_text\' -> only the translated text will be returned:\nresponse = client.translate_file(\n                        filepath=local_filepath_1,\n                        output_language="fr",\n                        return_type="raw_text"\n                    )\n\n# or return_type=\'url\' -> returns a link to translated file with original file\'s layout:\nresponse = client.translate_file(\n                        filepath=local_filepath_2,\n                        output_language="fr",\n                        return_type="url"\n                    )\n\nif response:\n    print(f"Translated File Response: {response}")\n```\n\n---\n\n## Web Endpoints\n\n### Web Search Request\n\nTo perform a **web search**:\n\n```python\nresponse = client.web_search(text="What is the capital of France?")\n\n# Or, if you want to specify the output_language and filter results\nresponse = client.web_search(\n                        text="What is the capital of France?",\n                        output_language="fr",\n                        desired_urls=["wikipedia.fr"]\n                    )\nif response:\n    print(f"Search Results: {response}")\n```\n\n---\n\n## LLM Endpoints\n\nLLM Endpoints provide a way to interact with several Large Language Models and Embedders in an unified way. Currently\nsupported `model`s are:\n\n| Chat Models          | Embedding Models       |\n| -------------------- | ---------------------- |\n| _gpt-3.5_ `Legacy`   | ada-v2                 |\n| gpt-4o               | text-embedding-3-large |\n| gpt-4o-mini          |                        |\n| command-r            |                        |\n| command-r-plus       |                        |\n| llama-3-70b-instruct |                        |\n| mistral-large        |                        |\n| mistral-small        |                        |\n| claude-3.5-sonnet    |                        |\n| claude-3-haiku       |                        |\n\n### 1. Chat Request\n\nTo **chat** with the LLM:\n\n```python\nresponse = client.llm_chat(text="Hello, how are you?")\n\n# Default model is handled, so that request is equivalent to:\nresponse = client.llm_chat(\n                        text="Hello, how are you?",\n                        model="gpt-4o-mini"\n                    )\nif response:\n    print(f"Chat Response: {response}")\n```\n\nThe list of **previous messages** can be provided through the `messages` parameter:\n\n```python\nresponse = client.llm_chat(\n                        text="What about uk?",\n                        messages=[\n                            {"role": "system", "content": "You are a helpful assistant."},\n                            {"role": "user", "content": "what is the capital city of spain?"},\n                            {"role": "assistant", "content": "The capital city of Spain is Madrid."},\n                        ],\n                        model="gpt-4o-mini",\n                    )\nif response:\n    print(f"Chat Response: {response}")\n```\n\n**Custom arguments** can be provided to the request, such as the dictionary `response_format` for the `chat` endpoint,\nor the `temperature` (in between 0 and 1):\n\n```python\nresponse = client.llm_chat(\n                        text="Hello, how are you? Respond in JSON format.",\n                        model="gpt-4o-mini",\n                        temperature=0.5,\n                        response_format={"type":"json_object"}\n                    )\nif response:\n    print(f"Chat Response: {response}")\n```\n\n### 2. Chat Stream\n\nIt is also possible to **stream** the response of the chat request:\n\n```python\nresponse = client.llm_chat_stream(text="Hello, how are you?")\n\n# Default model is handled, so that request is equivalent to:\nresponse = client.llm_chat_stream(\n                        text="Hello, how are you?",\n                        model="gpt-4o-mini"\n                    )\nif response:\n    print(f"Chat Response: {response}")\n\n```\n\nThe list of **previous messages** can be provided through the `messages` parameter as in the `chat` request (previous\nsection), as well as the `temperature` (in between 0 and 1) and other parameters:\n\n```python\nresponse = client.llm_chat_stream(\n                        text="What about uk?",\n                        messages=[\n                            {"role": "system", "content": "You are a helpful assistant."},\n                            {"role": "user", "content": "what is the capital city of spain?"},\n                            {"role": "assistant", "content": "The capital city of Spain is Madrid."},\n                        ],\n                        model="gpt-4o-mini",\n                        temperature=0.5,\n                    )\nif response:\n    print(f"Chat Response: {response}")\n```\n\nThe **response** in this case is a StreamingResponse, containing a generator which responses are similar to the\nfollowing, in order to keep compatibility with data stream protocols:\n\n```\n0: "\n\n0:"The"\n\n0:" capital"\n\n0:" city"\n\n0:" of"\n\n0:" the"\n\n0:" United"\n\n0:" Kingdom"\n\n0:" is"\n\n0:" London"\n\n0:"."\n\n2: "{\'id\': \'572a1c1e-ccc8-43bd-b4f1-3138016f7251\', \'choices\': [{\'delta\': {}, \'finish_reason\': \'stop\'}], \'request_id\': \'572a1c1e-ccc8-43bd-b4f1-3138016f7251\', \'response_id\': \'a1086bb8-16f7-4ca2-b27f-91bc8063e615\', \'status_code\': \'200\', \'status\': \'success\', \'message\': \'Chat response received.\\n(All the 3 previous `messages` have been read.)\', \'timestamp\': \'2025-02-14T09:18:32.032696+00:00\', \'cost\': \'1.8e-05\'}"\n\n```\n\n### 3. Embed Request\n\nTo **embed** some text using a LLM, using `ada-v2` model:\n\n```python\nresponse = client.llm_embed(text="Hello, how are you?", model="ada-v2")\nif response:\n    print(f"Embed Response: {response}")\n```\n\nOr using `text-embedding-ada-002` model:\n\n```python\nresponse = client.llm_embed(text="Hello, how are you?", model="text-embedding-ada-002")\nif response:\n    print(f"Embed Response: {response}")\n```\n\n---\n\n## Files Endpoints\n\n### Universal Reader and Parser\n\nThe Universal reader and parser allows to open many textual **file** formats and extract the content in a **standarized\nstructure**. In order to parse a file:\n\n```python\nlocal_filepath_1 = Path("/path/to/file1.docx")\nlocal_filepath_2 = Path("/path/to/file2.pdf")\n\nresponse = client.files_parse(filepath=local_filepath_1)\n\nif response:\n    print(f"Parsed File Response: {response}")\n```\n\nPrevious request can be further contolled by providing the **optional parameters**:\n\n```python\nresponse = client.files_parse(\n                        filepath=local_filepath_1,\n                        extract_type=chunks,\n                        read_images=True, # read images from the file, default is False\n                        k_min=500,\n                        k_max=1000,\n                        overlap=0,\n                        filter_pages="[1,2]", # subset of pages to select\n                    )\nif response:\n    print(f"Parsed File Response: {response}")\n```\n\n| extract_type     |                                                                                                            |\n| ---------------- | ---------------------------------------------------------------------------------------------------------- |\n| chunks `Default` | Returns the chunks of the file. You can custom its tokens size by setting `k_min`, `k_max`, `overlap`      |\n| subchunks        | Returns the subchunks of the file (minimal blocks in the file, usually containing around 20 or 30 tokens). |\n| pages            | Returns the content of the file parsed as pages                                                            |\n| file             | Returns the the whole file contents                                                                        |\n\n> 💡 **Tip:** When using `extract_type=chunks`, you can define the `k_min`, `k_max` and `overlap` parameters to control\n> the size of the chunks. Default values are `k_min=500`, `k_max=1200`, and `overlap=0`.\n\n### Files Index\n\nIndex group a set of files in order to be able to query them using natural language. The **Index attributes** are:\n\n| Attributes | Meaning                                                                                                                                        |\n| ---------- | ---------------------------------------------------------------------------------------------------------------------------------------------- |\n| index_uuid | Unique identifier of the index. It is randomly generated when the index is created and cannot be altered.                                      |\n| name       | Human-friendly name for the index, can be modified through the `rename_index` endpoint.                                                        |\n| created_at | Creation date                                                                                                                                  |\n| updated_at | Last operation performed in index                                                                                                              |\n| expires_at | Expiration date of the index. It will only be set once the `delete_index` request is explictly performed. (Default: None)                      |\n| status     | Status of the index. It will be `active`, and only when programmed for deletion it will be `countdown` (2h timeout before effective deletion). |\n| vectorized | Boolean status of the index. When `True`, the index is ready to be queried.                                                                    |\n| files      | List of files in the index. Contains their filehash, filename and size                                                                         |\n| storage    | Storage details of the index: total size in bytes and MB, number of files.                                                                     |\n|            |\n\nThe following **Index operations** are available:\n\n- `INDEX_LIST`: List all indexes.\n- `INDEX_DETAILS`: Get details of an index.\n- `INDEX_CREATE`: Create a new index and parse files.\n- `INDEX_ADD_FILES`: Add files to an existing index.\n- `INDEX_DELETE_FILES`: Delete files from an index.\n- `INDEX_DELETE`: Delete an index. **Warning**: _This is a delayed (2h) operation, allowed to be reverted with\n  `INDEX_RESTORE`. After 2h, the index will be **deleted and not recoverable**._\n- `INDEX_RESTORE`: Restore a deleted index _(within the 2h after it was marked for deletion)_.\n- `INDEX_EMBED`: Embed index contents.\n- `INDEX_ASK`: Ask a question to the index. It requires that `INDEX_EMBED` is performed to allow index contents\n  querying.\n\n### Files Index Requests\n\n#### 1. Existing Index Overview\n\nTo **list all indexes** in your organization, files included and storage details:\n\n```python\nresponse = client.files_index_list()\nif response:\n    print(f"List Indexes Response: {response}")\n```\n\nWith **get details** of an index you can see the list of files in the index, their filehashes, their size, the `status`\nof the index and the `vectorized` boolean status (find more details about the Index fields above):\n\n```python\nresponse = client.files_index_details(index_uuid="index-uuid")\nif response:\n    print(f"Index Details Response: {response}")\n```\n\n#### 2. Index Management\n\nTo **create a new index** and parse files, provide the list of **filepaths** you want to parse:\n\n```python\nlocal_filepaths = [Path("/path/to/file1.docx"), Path("/path/to/file2.pdf")]\n\nresponse = client.files_index_create(\n                        filepaths=local_filepaths,\n                        name="Cooking Recipes"\n                        read_images=True, # read images from the files, default is False\n                    )\nif response:\n    print(f"Index Create Response: {response}")\n```\n\nLet\'s say the new index has been created with the UUID `d55a285b-0a0d-4ba5-a918-857f63bc9063`. This UUID will be used in\nthe following requests, particularly in the `index_details` whenever some information about the index is needed.\n\nYou can **rename the index** with the `rename_index` method:\n\n```python\nindex_uuid = "d55a285b-0a0d-4ba5-a918-857f63bc9063"\nresponse = client.files_index_rename(\n                        index_uuid=index_uuid,\n                        name="Best Recipes"\n                    )\nif response:\n    print(f"Rename Index Response: {response}")\n```\n\nTo **add files** to an existing index, provide the list of **filepaths** you want to add:\n\n```python\nindex_uuid = "d55a285b-0a0d-4ba5-a918-857f63bc9063"\nlocal_filepath_3 = [Path("/path/to/file3.txt")]\n\nresponse = client.files_index_files_add(\n                        index_uuid=index_uuid,\n                        filepaths=local_filepath_3\n                        read_images=True, # read images from the files, default is False\n                    )\nif response:\n    print(f"Add Files to Index Response: {response}")\n```\n\nTo **delete files** from an existing index, specify the **filehashes** of the files you want to delete. You can see\nfilehashes contained in an index by requesting the index details. See a file deletion example below:\n\n```python\nindex_uuid = "d55a285b-0a0d-4ba5-a918-857f63bc9063"\nfilehashes_to_delete = ["2fa92ab4627c199a2827a363469bf4e513c67b758c34d1e316c2968ed68b9634"]\n\nresponse = client.files_index_files_delete(\n                        index_uuid=index_uuid,\n                        files_hashes=filehashes_to_delete\n                    )\nif response:\n    print(f"Delete Files from Index Response: {response}")\n```\n\nTo **delete an index** (it will be marked for deletion which will become effective **after 2h**):\n\n```python\nresponse = client.files_index_delete(index_uuid="index-to-delete-uuid")\nif response:\n    print(f"Delete Index Response: {response}")\n```\n\nTo **restore an index** marked for deletion (only possible during the 2h after the `INDEX_DELETE` was requested):\n\n```python\nresponse = client.files_index_restore(index_uuid="index-to-restore-uuid")\nif response:\n    print(f"Restore Index Response: {response}")\n```\n\n#### 3. Index Querying\n\nTo **embed** or **vectorize index contents** in order to allow the query operations:\n\n```python\nresponse = client.files_index_embed(index_uuid="index-uuid")\nif response:\n    print(f"Embed Index Response: {response}")\n```\n\nTo **ask a question** about the index documents (it requires that your `index.status.vectorized` is set to `True`):\n\n```python\nresponse = client.files_index_ask(\n                        index_uuid="index-uuid",\n                        question="What is Delos?"\n                    )\nif response:\n    print(f"Ask Index Response: {response}")\n```\n\nYou can sub-select some of the files in the index through the `active_files` parameter, which can receive a list of\nfilehashes to be queried, or the special words `all` (default behavior) or `none`. See some examples of valid\n`active_files` values below:\n\n| active_files                     | Explanation                                            |\n| -------------------------------- | ------------------------------------------------------ |\n| "abc123def456"                   | Perform the research on a single index file            |\n| ["abc123def456", "789ghi012jkl"] | A list of files                                        |\n| None                             | Not provided                                           |\n| "all" (default)                  | Query the subselection of all files available in index |\n| "none"                           | Query none of the available files                      |\n\nFor example, to ask the question using all the files in the index:\n\n```python\nresponse = client.files_index_ask(\n                        index_uuid="index-uuid",\n                        question="What is Delos?"\n                        active_files="all"\n                    )\nif response:\n    print(f"Ask Index Response: {response}")\n```\n\nOr only on some of the files:\n\n```python\nresponse = client.files_index_ask(\n                        index_uuid="index-uuid",\n                        question="What is Delos?",\n                        active_files=["abc123def456", "789ghi012jkl"]\n                    )\nif response:\n    print(f"Ask Index Response: {response}")\n```\n\n> **Hint:** You can find the filehashes of the files contained in the index by using the `files_index_details` method.\n\n## Requests Usage and Storage\n\nAll request responses show the **number of tokens** and **cost** consumed by the request. The **storage** for index\ndocuments is **limited** up to your organization\'s quota and is shared between all indexes within your organization.\nContents **do not expire**, but they can be deleted by performing an explicit request through the API endpoints or\nthrough the **DelosPlatform** at `https://platform.delos-suite.ai/`.\n\nIn the **DelosPlatform**, you can monitor the requests performed by your organization with your API Key and the files\nstored in the Index Storage.\n\n![API key usage in Delos Platform](https://i.ibb.co/VTD35z1/api-key-usage.png)\n\nThrough both the native requests towards Delos and the Python client, you can handle and delete files directly from the\nDelos Platform.\n',
    'author': 'Maria',
    'author_email': 'mariaibanez@delosintelligence.fr',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'py_modules': modules,
    'install_requires': install_requires,
    'python_requires': '>=3.11,<4.0',
}


setup(**setup_kwargs)



"""Exceptions while using DelosClient."""


class APIKeyMissingError(Exception):
    """Exception raised when an API key is missing."""

    def __init__(self) -> None:
        """Initialize the exception with a message."""
        message = "API Key for Delos is missing."
        super().__init__(message)


class ResponseFormatTypeMissingError(Exception):
    """Exception raised when a response format is missing a type."""

    def __init__(self) -> None:
        """Initialize the exception with a message."""
        message = "The `response_format` dictionary must contain a `type` key."
        super().__init__(message)


class InvalidInputError(Exception):
    """Exception raised when invalid input is provided."""

    def __init__(self, message: str) -> None:
        """Initialize the exception with a message."""
        super().__init__(message)



"""Data structure for the different versions of the API."""

from datetime import UTC, datetime
from enum import Enum
from typing import Any

from pydantic import BaseModel

from .endpoints import DelosEndpoints, Endpoints


class Release(Enum):
    """Enum for the different versions of the API."""

    V1 = "v1"


class DelosClientReleases(BaseModel):
    """Data structure for the different versions of the API."""

    version: Release
    release_date: datetime
    details: dict[str, Any]
    suffix: str
    available_endpoints: list[DelosEndpoints]


first_release = DelosClientReleases(
    version=Release.V1,
    details={
        "version": "v1",
        "release_date": "2024-11-20",
        "description": "Initial release of the API",
    },
    release_date=datetime(year=2024, month=11, day=20, tzinfo=UTC),
    suffix="/v1",
    available_endpoints=[
        Endpoints.STATUS.HEALTH,
        Endpoints.TRANSLATE.TRANSLATE_TEXT,
        Endpoints.TRANSLATE.TRANSLATE_FILE,
        Endpoints.WEB.SEARCH,
        Endpoints.FILES.PARSER,
        Endpoints.FILES.INDEX_CREATE,
        Endpoints.FILES.INDEX_ADD_FILES,
        Endpoints.FILES.INDEX_DELETE_FILES,
        Endpoints.FILES.INDEX_DELETE,
        Endpoints.FILES.INDEX_RESTORE,
        Endpoints.FILES.INDEX_ASK,
        Endpoints.FILES.INDEX_EMBED,
        Endpoints.FILES.INDEX_LIST,
        Endpoints.FILES.INDEX_DETAILS,
        Endpoints.LLM.CHAT,
        Endpoints.LLM.EMBED,
    ],
)

AllReleases: list[DelosClientReleases] = []
AllReleases.append(first_release)
